#NoTrayIcon
#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_icon=DSTunes.ico
#AutoIt3Wrapper_UseUpx=n
#AutoIt3Wrapper_UseX64=n
#AutoIt3Wrapper_Res_Comment=DSTunes
#AutoIt3Wrapper_Res_Description=The DSi Music Converter
#AutoIt3Wrapper_Res_Fileversion=0.9.1.1
#AutoIt3Wrapper_Res_LegalCopyright=2010 Michael Michta
#AutoIt3Wrapper_Res_Language=1033
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
#NoAutoIt3Execute
#include <ButtonConstants.au3>
#include <EditConstants.au3>
#include <GUIConstantsEx.au3>
#include <ProgressConstants.au3>
#include <WindowsConstants.au3>
#include<File.au3>
#include<Misc.au3>
#include<String.au3>
#include<Array.au3>

;;Declare variables and set options
Const $VerInfo = "0.9.1.1 Alpha"
Dim $OutFile, $InFile, $Cancel, $PID, $Current, $Total, $Mode
Opt("GUIOnEventMode", 1)

;;Ensure only one instance of DSTunes is running
If Not _Singleton("DSTunesMT-E7F13B6703EBF9F32E20640E7C035162", 1) Then Quit("Singleton")

;;Parse arguments
If $CmdLine[0] = 0 Then Quit("NoArgs")
Select
	Case StringRight($CmdLine[1], 4) = ".mp3" ;File mode
		$Mode = "File"
		Dim $List[1]
		For $X = 1 To $CmdLine[0]
			If StringRight($CmdLine[$X], 4) = ".mp3" Then
				_ArrayAdd($List, $CmdLine[$X])
				$List[0] += 1
			Else
				Quit("Syntax")
			EndIf
		Next
	Case Else ;Folder mode
		$Mode = "Folder"
		If $CmdLine[0] > 1 Then Quit("Syntax")

		;;Get Directories
		$InDir = $CmdLine[1]
		$OutDir = $InDir & "-AAC"

		;;Build file list
		$List = _FileListToArray($InDir, "*.mp3", 1)
		If Not IsArray($List) Then Quit("NoMp3")
EndSelect

;;Create $OutDir if it does not exist
If ($Mode = "Folder") and (Not FileExists($OutDir)) Then DirCreate($OutDir)

;;Extract FFMPEG
FileInstall("ffmpeg.exe", @TempDir & "\DST-FFMPEG.exe", 1)
If Not FileExists(@TempDir & "\DST-FFMPEG.exe") Then Quit("InitFailed")

;;Create GUI
$GUI = GUICreate("DSTunes V" & $VerInfo, 626, 161)
$ButtonCancel = GUICtrlCreateButton("Cancel", 536, 128, 81, 25, $WS_GROUP)
$InputStatus = GUICtrlCreateInput("", 14, 104, 601, 21, BitOR($ES_CENTER, $ES_AUTOHSCROLL, $ES_READONLY))
$ProgressFile = GUICtrlCreateProgress(12, 21, 601, 17)
GUICtrlCreateLabel("Current File:", 12, 3, 60, 17)
GUICtrlCreateLabel("Total Progress:", 10, 42, 75, 17)
$ProgressTotal = GUICtrlCreateProgress(10, 60, 601, 17)

;;Hook Events
GUICtrlSetOnEvent($ButtonCancel, "Cancel")
GUISetOnEvent($GUI_EVENT_CLOSE, "Cancel")

;;Show GUI
GUISetState(@SW_SHOW)

;;Start Timer
$Timer = TimerInit()

;;Convert Files
For $X = 1 To $List[0]

	;;Get file names
	Switch $Mode
		Case "Folder"
			$InFile = $InDir & "\" & $List[$X]
			$OutFile = $OutDir & "\" & StringTrimRight($List[$X], 3) & "m4a"
		Case "File"
			$InFile = $List[$X]
			$OutFile = StringTrimRight($List[$X], 3) & "m4a"
	EndSwitch

	;;Build command string
	$Command = StringFormat(@TempDir & '\DST-FFMPEG.exe -y -strict experimental -i "%s" -ab %s -map_meta_data "%s":"%s" "%s"', $InFile, "128k", $InFile, $OutFile, $OutFile)

	;;Execute FFMPEG to do the heavy lifting
	$PID = Run($Command, @ScriptDir, @SW_HIDE, 0x6)
	;Get Duration of track
	While ProcessExists($PID)
		$Read = StderrRead($PID)
		If StringInStr($Read, "Duration:") Then
			$Between = _StringBetween($Read, "Duration:", ",")
			$Split = StringSplit($Between[0], ":")
			$Duration = $Split[1] * 60 * 60
			$Duration += $Split[2] * 60
			$Duration += $Split[3]
			ExitLoop
		EndIf
		Sleep(100)
	WEnd

	;;Keep GUI updated with progress info
	While ProcessExists($PID)
		$Read = StderrRead($PID)
		If StringInStr($Read, "time=") Then
			$Between = _StringBetween($Read, "time=", " ")
			$Current = $Between[UBound($Between) - 1]
		EndIf
		$PercentTotal = Round(($X - 1) / $List[0] * 100)
		$PercentFile = Round($Current / $Duration * 100)
		GUICtrlSetData($ProgressTotal, $PercentTotal)
		GUICtrlSetData($ProgressFile, $PercentFile)
		GUICtrlSetData($InputStatus, $PercentTotal & '%:Converting "' & $List[$X] & '" (' & $X & "\" & $List[0] & ")")
		Sleep(100)
	WEnd
Next

;;Display msgbox after conversion is complete
$Time = TimerDiff($Timer) / 1000
$Minutes = Int($Time / 60)
$Seconds = Int(Mod($Time, 60))
$Text = StringFormat("Converted %s file(s) in %s:%s", $List[0], $Minutes, $Seconds)
MsgBox(64, "DSTunes", $Text, Default, $GUI)

;;Cancel Event Handler
Func Cancel()
	ProcessClose($PID)
	ProcessWaitClose($PID)
	FileDelete($OutFile)
	Exit
EndFunc   ;==>Cancel

;;Error Handler
Func Quit($Reason)
	Switch $Reason
		Case "Syntax"
			MsgBox(48, "DSTunes", "Syntax:" & @CRLF & "DSTunes <Folder>" & @CRLF & "DSTunes <MP3-1> ... <MP3-64>")
		Case "NoArgs"
			MsgBox(64, "DSTunes", "Drag a folder or mp3 files onto the icon to begin conversion.")
		Case "InitFailed"
			MsgBox(48, "DSTunes", "Initialization Failed.")
		Case "Singleton"
			MsgBox(64, "DSTunes", "Only one instance of DSTunes can be executed at a time.")
		Case "NoMP3"
			MsgBox(48, "DSTunes", "No MP3 files found.")
	EndSwitch
	Exit
EndFunc   ;==>Quit